@@ -38,22 +38,32 @@ model.compile(optimizer=Adam(LR), loss=similarity_loss)
 history = model.fit(train_ds)
 ```
 """
-
 from collections import defaultdict
 from copy import copy
 from pathlib import Path
-from typing import DefaultDict, Dict, List, MutableMapping, MutableSequence
-from typing import Optional, Sequence, Union
+from typing import (
+    DefaultDict,
+    Dict,
+    List,
+    MutableMapping,
+    MutableSequence,
+    Optional,
+    Sequence,
+    Union,
+)
+
 import numpy as np
 from tabulate import tabulate
 import tensorflow as tf
+from tqdm.auto import tqdm
 from tensorflow.keras.optimizers import Optimizer
 from tensorflow.keras.metrics import Metric
 from tensorflow.keras.losses import Loss
-from tqdm.auto import tqdm
 
 from tensorflow_similarity.classification_metrics import ClassificationMetric
-from tensorflow_similarity.classification_metrics import make_classification_metric  # noqa
+from tensorflow_similarity.classification_metrics import (
+    make_classification_metric,
+)
 from tensorflow_similarity.distances import Distance
 from tensorflow_similarity.distances import distance_canonicalizer
 from tensorflow_similarity.training_metrics import DistanceMetric
@@ -78,23 +88,20 @@ class SimilarityModel(tf.keras.Model):
     core features.
     """
 
-    # @property
-    # def _index(self):
-    #     if not hasattr(self, '_index'):
-    #         ValueError("Index doesn't exist: index data before quering it")
-    #     return self._index
-
-    # @index.setter
-    # def _index(self, index):
-    #     self._index: Indexer = index
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
 
     def compile(
         self,
         optimizer: Union[Optimizer, str, Dict, List] = "rmsprop",
         loss: Optional[Union[Loss, MetricLoss, str, Dict, List]] = None,
-        metrics: Optional[Union[Metric, DistanceMetric, str, Dict, List]] = None,  # noqa
+        metrics: Optional[
+            Union[Metric, DistanceMetric, str, Dict, List]
+        ] = None,  # noqa
         loss_weights: Optional[Union[List, Dict]] = None,
-        weighted_metrics: Optional[Union[Metric, DistanceMetric, str, Dict, List]] = None,  # noqa
+        weighted_metrics: Optional[
+            Union[Metric, DistanceMetric, str, Dict, List]
+        ] = None,  # noqa
         run_eagerly: bool = False,
         steps_per_execution: int = 1,
         distance: Union[Distance, str] = "auto",
@@ -103,7 +110,7 @@ class SimilarityModel(tf.keras.Model):
         search: Union[Search, str] = "nmslib",
         evaluator: Union[Evaluator, str] = "memory",
         stat_buffer_size: int = 1000,
-        **kwargs
+        **kwargs,
     ):
         """Configures the model for training.
 
@@ -118,7 +125,7 @@ class SimilarityModel(tf.keras.Model):
               [tf.keras.losses](https://www.tensorflow.org/api_docs/python/tf/keras/losses)
               for the losses available directly in TensorFlow.
 
-              metrics: List of metrics to be evaluated by the model during
+            metrics: List of metrics to be evaluated by the model during
               training and testing. Each of those can be a string, a function or a
               [tensorflow_similairty.metrics.*](../metrics.md) instance. Note that
               the metrics used for some type of metric-learning such as distance
@@ -174,6 +181,12 @@ class SimilarityModel(tf.keras.Model):
             distance: Distance used to compute embeddings proximity.  Defaults
               to 'cosine'.
 
+            embedding_output: Which model output head predicts the embeddings
+              that should be indexed. Default to None which is for single output
+              model. For multi-head model, the callee, usually the
+              `SimilarityModel()` class is responsible for passing the correct
+              one.
+
             kv_store: How to store the indexed records.  Defaults to 'memory'.
 
             search: Which `Search()` framework to use to perform KNN search.
@@ -182,12 +195,6 @@ class SimilarityModel(tf.keras.Model):
             evaluator: What type of `Evaluator()` to use to evaluate index
               performance. Defaults to in-memory one.
 
-            embedding_output: Which model output head predicts the embeddings
-              that should be indexed. Default to None which is for single output
-              model. For multi-head model, the callee, usually the
-              `SimilarityModel()` class is responsible for passing the correct
-              one.
-
             stat_buffer_size: Size of the sliding windows buffer used to compute
               index performance. Defaults to 1000.
 
@@ -205,8 +212,10 @@ class SimilarityModel(tf.keras.Model):
             try:
                 distance = metric_loss.distance
             except AttributeError:
-                msg = ("distance='auto' only works if the first loss is a "
-                       "metric loss")
+                msg = (
+                    "distance='auto' only works if the first loss is a "
+                    "metric loss"
+                )
 
                 raise ValueError(msg)
             print(
@@ -235,9 +244,24 @@ class SimilarityModel(tf.keras.Model):
             weighted_metrics=weighted_metrics,
             run_eagerly=run_eagerly,
             steps_per_execution=steps_per_execution,
-            **kwargs
+            **kwargs,
         )
 
+    @property
+    def _index(self) -> Indexer:
+        if not hasattr(self, "_ix") and self._ix:
+            raise AttributeError(
+                "The model does not currently have an index. To create an "
+                "index you must call either model.compile() or "
+                "model.create_index() and set a valid Distance."
+            )
+
+        return self._ix
+
+    @_index.setter
+    def _index(self, index: Indexer) -> None:
+        self._ix = index
+
     def create_index(
         self,
         distance: Union[Distance, str] = "cosine",
@@ -325,7 +349,7 @@ class SimilarityModel(tf.keras.Model):
 
             y: class ids associated with the data if any. Defaults to None.
 
-            store_data: store the data associated with the samples in the key
+            data: store the data associated with the samples in the key
             value store. Defaults to True.
 
             build: Rebuild the index after indexing. This is needed to make the
@@ -336,11 +360,6 @@ class SimilarityModel(tf.keras.Model):
             verbose: Output indexing progress info. Defaults to 1.
         """
 
-        if not self._index:
-            raise Exception(
-                "You need to compile the model with a valid"
-                "distance to be able to use the indexing"
-            )
         if verbose:
             print("[Indexing %d points]" % len(x))
             print("|-Computing embeddings")
@@ -354,12 +373,14 @@ class SimilarityModel(tf.keras.Model):
             verbose=verbose,
         )
 
-    def index_single(self,
-                     x: Tensor,
-                     y: IntTensor = None,
-                     data: Optional[Tensor] = None,
-                     build: bool = True,
-                     verbose: int = 1):
+    def index_single(
+        self,
+        x: Tensor,
+        y: IntTensor = None,
+        data: Optional[Tensor] = None,
+        build: bool = True,
+        verbose: int = 1,
+    ):
         """Index data.
 
         Args:
@@ -378,20 +399,19 @@ class SimilarityModel(tf.keras.Model):
             verbose: Output indexing progress info. Defaults to 1.
         """
 
-        if not self._index:
-            raise Exception('You need to compile the model with a valid'
-                            'distance to be able to use the indexing')
         if verbose:
-            print('[Indexing 1 point]')
-            print('|-Computing embeddings')
+            print("[Indexing 1 point]")
+            print("|-Computing embeddings")
 
         x = tf.expand_dims(x, axis=0)
         prediction = self.predict(x)
-        self._index.add(prediction=prediction,
-                        label=y,
-                        data=data,
-                        build=build,
-                        verbose=verbose)
+        self._index.add(
+            prediction=prediction,
+            label=y,
+            data=data,
+            build=build,
+            verbose=verbose,
+        )
 
     def lookup(
         self, x: Tensor, k: int = 5, verbose: int = 1
@@ -443,7 +463,8 @@ class SimilarityModel(tf.keras.Model):
         calibration_metric: Union[str, ClassificationMetric] = "f1",
         matcher: Union[str, ClassificationMatch] = "match_nearest",
         extra_metrics: MutableSequence[Union[str, ClassificationMetric]] = [
-            "precision", "recall"
+            "precision",
+            "recall",
         ],  # noqa
         rounding: int = 2,
         verbose: int = 1,
@@ -599,8 +620,10 @@ class SimilarityModel(tf.keras.Model):
             IndexError: Index must contain embeddings but is currently empty.
         """
         if self._index.size() == 0:
-            raise IndexError("Index must contain embeddings but is "
-                             "currently empty. Have you run model.index()?")
+            raise IndexError(
+                "Index must contain embeddings but is "
+                "currently empty. Have you run model.index()?"
+            )
 
         # get embeddings
         if verbose:
@@ -631,7 +654,8 @@ class SimilarityModel(tf.keras.Model):
         y: IntTensor,
         k: int = 1,
         extra_metrics: MutableSequence[Union[str, ClassificationMetric]] = [
-            "precision", "recall"
+            "precision",
+            "recall",
         ],  # noqa
         matcher: Union[str, ClassificationMatch] = "match_nearest",
         verbose: int = 1,
@@ -669,8 +693,10 @@ class SimilarityModel(tf.keras.Model):
         # There is some code duplication in this function but that is the best
         # solution to keep the end-user API clean and doing inferences once.
         if self._index.size() == 0:
-            raise IndexError("Index must contain embeddings but is "
-                             "currently empty. Have you run model.index()?")
+            raise IndexError(
+                "Index must contain embeddings but is currently empty. Have "
+                "you added examples via model.index()?"
+            )
 
         if not self._index.is_calibrated:
             raise ValueError("Uncalibrated model: run model.calibration()")
@@ -768,6 +794,7 @@ class SimilarityModel(tf.keras.Model):
         compression: bool = True,
         overwrite: bool = True,
         include_optimizer: bool = True,
+        save_format: Optional[str] = None,
         signatures=None,
         options=None,
         save_traces: bool = True,
@@ -785,6 +812,10 @@ class SimilarityModel(tf.keras.Model):
 
             include_optimizer: Save optimizer state. Defaults to True.
 
+            save_format: Either 'tf' or 'h5', indicating whether to save the
+            model to Tensorflow SavedModel or HDF5. Defaults to 'tf' in
+            TF 2.X, and 'h5' in TF 1.X.
+
             signatures: Signatures to save with the model. Defaults to None.
 
             options: A `tf.saved_model.SaveOptions` to save with the model.
@@ -798,23 +829,31 @@ class SimilarityModel(tf.keras.Model):
             custom layers/models implement a get_config() method.
         """
 
-        # save trace doesn't exist prior to 2.4 -- asking for it but not
-        # using it
-
         # call underlying keras method to save the mode graph and its weights
-        tf.keras.models.save_model(
-            self,
+        super().save(
             filepath,
             overwrite=overwrite,
             include_optimizer=include_optimizer,
+            save_format=save_format,
             signatures=signatures,
             options=options,
             save_traces=save_traces,
         )
-        if hasattr(self, "_index") and self._index and save_index:
+
+        if hasattr(self, "_ix") and self._index and save_index:
             self.save_index(filepath, compression=compression)
         else:
-            print("Index not saved as save_index=False")
+            msg = "The index was not saved with the model."
+            if not hasattr(self, "_ix"):
+                msg = msg + (
+                    "The model does not currently have an index. To use indexing "
+                    "you must call either model.compile() or model.create_index() "
+                    "and set a valid Distance.")
+
+            if not save_index:
+                msg = msg + " The save_index param is set to False."
+
+            print(msg)
 
     def to_data_frame(self, num_items: int = 0) -> PandasDataFrame:
         """Export data as pandas dataframe
